from collections import defaultdict
from datetime import datetime as dt
from functools import wraps
import os
import pandas as pd
import quilt3 as qt
from polly.auth import Polly
from polly.omixatlas import OmixAtlas
from pydantic import StrictFloat, StrictInt, StrictStr, create_model

# Obtain authentication tokens
AUTH_TOKEN = 'eyJjdHkiOiJKV1QiLCJlbmMiOiJBMjU2R0NNIiwiYWxnIjoiUlNBLU9BRVAifQ.mJokdEXE0zEyEROycyck0P8WtXbU9DdpnmMU8HqNia6-pQMrLEpZIpOxi3-fehY0rUlNe-S6pTqbVd7_kDyfl_SqprXz-zYWBs55dyN60Z0j252qghoMvcu08TM9OvrI3y4YEeCgmcYpg3oI6foRzynAnMVJSzOlqxv4P_yYX1XRB6WQBgFVryXGc2QglwgCy9ng5gRskLOP9fAnQ1jwowTE402uy2yrC5_Hu8FaHZHxw4f-UIiE5lBN99Ajkva0J1q10VRAWUdWkQhBvG5Fu2gq2r3YnLJydtJgfLXpEmx20EmS3zGc1rJ9sTdarndzpnSj0ZvECItRzM0shR0rdw.rmzroqG6mj8yni70.UUfpevy_y2UA0bq-98Q6q0PFUJJD2gkPsyp9WIJP0V50RS14zeD7B_JzKExTwWSlZYe4pmwTbwvEkmJX_jIVlFL2rLkwdJtORbalKdVoC_tpC4LkI7kJQJnZLlZAG1O03xqQtWHcFL33sYQFuX_jrrMeZI-KX_tc_0t3cVGwvvkeYhg9mYhYRTGcZ7oSHCHuBLEcDTlcKFayxoECBV5PjloKdeedrr7QL2q7-c4r5wRxCNd7J9xyAZXJICShIYMSqjLFk17LisTfjrNvptLo6vVUCQhbgmh3oK_t7k71rz6sXMROq_1nPZN7aXM-K8MAtlbgrzPbASM5x5dS6TrQ44L_L2HbrYsuq8KeBxhW71tZ2h85xIz2gwPD53MBohGp40pEvm22ZXiAiF1I5t8JTgxXtpDE4YHPJO1M-G_lT8bXFg1nt0rqsAIySZNcyOdIPg-JnpJNTaKbMQzQi3mCK7nIlRsoqk50GAm8BgnnQOSnBymOu0xWip6Y_wGd14Q3hMa_JXUoxnxqLhFhWadMm9RCqyXMeEOAPbIkypNasc6zDmH6mzdOjEtzz0VhHOvtP5xcabFS1sCZrv7dhAE_S9jimQwOTcb2M91bEl-QcOwy4eujYjcJuQfHS0968EJmNSgjjgjySNuA-FI49zApJPiqqUzkfZmid9sMe3PfLselnWyLfKJ5cynbmQ0EiB4DfVbSgIjohN1xCk76DdO5DC_Z6vrGQhdAtEoZhEidp89_g8sAXrlsCIl2hQySq3F-vyZB78am9ZTF7nO9Hvwy89-OQBw9603aIblFRMlAYhxagL2Fa5N_KojMvHGgD2lJ7z7fXpSlXnjzUl5ZpJmkZZEqZLH4yQUd5aYCLVw2cciLnU7P8KKoFooqWDYj_zPm2yYdGgsEMcg5s-Zr1bCnMBfXPaLNeI99Bi7o834NkPNCUPSFEiPRMiLBhjWcGon5lZJQg8kH7SsSERL16k5GaYAJAbWmWK3khb-5N9n2lpeCGfWDS3x9bw1jDY0lbdU8HZpuDIueWibxnvQfgdwYRtOI0fZAvHgrJDH7Rm1-fslJuChSw96zCALsdTdpgVPNh-2563p7ghiTssQJh1Y93widuwL6XqVmWGa1SSRDs0iy6Fv1zU4vSFXJgfGUfgUPhwE-fETFC8uFtWjQjFnkeLh1--alGIRanma6YtQOKKxfAGAKhi9NmFHGsEM-HLbeTenq77mgnCno75JUdwNXWIVStZMIrp1NE5rn3g1I4nSncSJnXcftX2FbxSPVDFvqToYUfZunvFFqzc0co5DkXuO91YBe5SvZ3AQWFhP9vAysfSWWR_AaDSJEUHI.bmmjm89D_jxRM1KA-9tyJg'
# Polly.auth(os.environ['AUTH_TOKEN'])
Polly.auth(AUTH_TOKEN)

# Defining OMIXATLAS object
OMIXATLAS = OmixAtlas()

DATATYPE_MAP = {'string': StrictStr,
                'bigint': StrictInt,
                'integer': StrictInt,
                'double': StrictFloat}


def timeit(func):
    """
    Timing function to time error collection.
    Args:
        func:

    Returns:
    """

    @wraps(func)
    def timed(*args, **kw):
        tstart = dt.now()
        output = func(*args, **kw)
        tend = dt.now()
        print('"{}" took {} to execute.\n'.format(func.__name__, (tend - tstart)))
        return output

    return timed


def dataset_metadata_validator_construct_args_for_pool(repos, bucket):
    """
    Given a list of repos and bucket, it creates a dict of arguments for process workers
    Args:
        repos: list of repos for workers to work on
        bucket: s3 bucket

    Returns:
        args_for_pool [dict]: list of arguments against a repo for the workers.
    """
    args_for_pool = defaultdict(list)
    for repo in repos:
        print(f'Getting manifest for: {repo}')
        pkg = qt.Package.browse(repo, bucket)
        for key, entry in pkg.walk():
            args_for_pool[repo].append((repo, key, entry.meta))
    return args_for_pool


def sample_metadata_validator_construct_args_for_pool(repos):
    """

    Args:
        repos:

    Returns:

    """
    args_for_pool = defaultdict(list)
    for repo in repos:
        # print(f'Getting {repo} samples\' schema...')
        # desc_df = get_query_results(f'DESCRIBE {repo}.samples')
        # print(f'Building validator schema...')
        # schema = build_validator_schema(desc_df)
        print(f'Getting samples for: {repo}')
        t_start = dt.now()
        samples = get_query_results(f'SELECT * FROM {repo}.samples')
        print(f'Time taken to get the query: {dt.now() - t_start}')
        for val in samples.to_dict('records'):
            args_for_pool[repo].append((repo, val))
    return args_for_pool


def list_to_set_in_dict(obj):
    """
    Recursively convert list values inside a dic to sets
    Args:
        obj: Current value being checked

    Returns:
        obj: Dict with lists converted to sets.
    """
    if isinstance(obj, list):
        return {list_to_set_in_dict(v) for v in obj}
    elif isinstance(obj, dict):
        return {k: list_to_set_in_dict(v) for k, v in obj.items()}
    return obj


def lower_and_strip(obj):
    """
    Lower and strip all values in a list
    Args:
        obj:

    Returns:

    """
    for key, val in obj.items():
        if isinstance(val, list):
            obj[key] = [v.lower().strip() for v in val]
        elif isinstance(val, dict):
            obj[key] = lower_and_strip(obj[key])
    return obj


def get_query_results(query):
    """
    Given a query for Polly-python, return the resultant DF
    Args:
        query: SQL Query

    Returns:
        DataFrame: Response to the query
    """
    return OMIXATLAS.query_metadata(query, query_api_version="v2")


def get_all_omix_atlas():
    """
    Function to get list of all
    Returns:

    """
    return pd.DataFrame.from_dict(OMIXATLAS.get_all_omixatlas()['data'])


def build_validator_schema(desc_df):
    """
    Takes a description DF of a sample and returns a pydantic class with the values
    Args:
        desc_df: Result of "DESC

    Returns:

    """
    schema_dict = dict(zip(desc_df['column_name'], desc_df['column_type']))
    for k, v in schema_dict.items():
        schema_dict[k] = (DATATYPE_MAP[v], ...)
    schema = create_model('Schema', **schema_dict)
    return schema


if __name__ == '__main__':
    test_schema_df = pd.DataFrame.from_dict(
        {'column_type': ['string', 'integer', 'string'], 'column_name': ['a', 'b', 'c']})
    test_schema = build_validator_schema(test_schema_df)
    test_schema(**{'a': '67', 'b': 987, 'c': 'abs'})
